#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/resource.h>

#include "srv.h"

enum action { start, stop, restart, reload, status };

int main (int argc, char *argv[])
{
	struct service service;
	enum action action;
	char *argv2;
	int i;
	pid_t pid;
	int retstatus;

	if (argc != 3) {
		fprintf(stderr, "usage: service servicename action\n");
		exit(1);
	}

	if (service_read(argv[1], &service) < 0) {
		fprintf(stderr, "invalid service\n");
		exit(1);
	}

	if (service.script == NULL || service.script[0] == '\0') {
		fprintf(stderr, "no script defined\n");
		exit(1);
	}
	if (service.script[0] != '/') {
		fprintf(stderr, "script needs an absolute pathname\n");
		exit(1);
	}

	if (!strcmp(argv[2], "start")) {
		action = start;
	} else if (!strcmp(argv[2], "stop")) {
		action = stop;
	} else if (!strcmp(argv[2], "restart")) {
		action = restart;
	} else if (!strcmp(argv[2], "reload") && service.supportsreload) {
		action = reload;
	} else if (!strcmp(argv[2], "force-reload")) {
		if (service.supportsreload) {
			action = reload;
		} else {
			action = restart;
		}
	} else if (!strcmp(argv[2], "status")) {
		action = status;
	} else {
		fprintf(stderr, "Invalid action. Valid actions are:\n");
		fprintf(stderr, "start stop restart %sforce-reload status\n",
		  service.supportsreload ? "reload " : "");
		exit(1);
	}

	setpriority(PRIO_PROCESS, 0, service.nice);

	setenv("RUNNING_VIA_SERVICE", "1", 1);

	argv2 = argv[2];

	if (action == restart) {
		pid = fork();
		if (pid < 0) {
			fprintf(stderr, "fork failed()\n");
			exit(1);
		}
		if (pid == 0) {
			execl(service.script, service.script, "stop", NULL);
			fprintf(stderr, "execl failed\n");
			exit(1);
		}
		while (1) {
			i = waitpid(pid, &retstatus, 0);
			if (i < 0) {
				fprintf(stderr, "waitpid failed\n");
				exit(1);
			}
			if (i == pid) break;
		}
		if (!WIFEXITED(retstatus)) exit(1);
		if (WEXITSTATUS(status) != 0) exit(WEXITSTATUS(status));
		argv2 = "start";
	}

	execl(service.script, service.script, argv2, NULL);
	fprintf(stderr, "execl failed\n");
	exit(1);
}
